#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_SMBUS
/* assume SMBus ioctl support, only for FreeBSD */

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ioctl.h>

#include <machine/smb.h>

#include "methods.h"

/* SMBus Base Address, global */
int smb_base	= 2002;		/* arbitrary positive number */
int smb_slave	= LM_ADDR;
int smb_wbtemp1	= WBtemp1_ADDR;
int smb_wbtemp2	= WBtemp2_ADDR;

/* SMBus device file name, global */
extern char *smb_devfile;

static int iosmb;
static char buf[128];

static int OpenIO()
{
	char byte;
	struct smbcmd cmd;
	cmd.slave = smb_slave;
	cmd.data.byte_ptr = &byte;

	if ((iosmb = open(smb_devfile, 000)) < 0) {
		strcpy(buf, "ioctl(");
		strcat(buf, smb_devfile + 5);
		strcat(buf, ":open)");
		perror(buf);
		return -1;
	}
/* command Reg:0x47 is not universal for all hardware monitor chips */
#if 0
	cmd.cmd = 0x47;
	if (ioctl(iosmb, SMB_READB, &cmd) == -1) {
		strcpy(buf, "ioctl(");
		strcat(buf, smb_devfile + 5);
		strcat(buf, ":read Reg.0x47)");
		perror(buf);
		return -1;
	}
#endif
	return 0;
}

static void CloseIO()
{
	close(iosmb);
}

static int ReadByte(int addr)
{
	struct smbcmd cmd;
	char ret;
	cmd.slave = smb_slave;
	cmd.cmd = addr;
	cmd.data.byte_ptr = &ret;
	if (ioctl(iosmb, SMB_READB, &cmd) == -1) {
/*		strcpy(buf, "ioctl(");
		strcat(buf, smb_devfile + 5);
		strcat(buf, ":readbyte)");
		perror(buf);
		exit(-1); */
		ret = 0xFF;
	}
	return (int) (ret & 0xFF);
}

static void WriteByte(int addr, int value)
{
	struct smbcmd cmd;
	cmd.slave = smb_slave;
	cmd.cmd = addr;
	cmd.data.byte = value;
	if (ioctl(iosmb, SMB_WRITEB, &cmd) == -1) {
		strcpy(buf, "ioctl(");
		strcat(buf, smb_devfile + 5);
		strcat(buf, ":writebyte)");
		perror(buf);
		exit(-1);
	}
}

static int ReadTemp1()
{
	struct smbcmd cmd;
	short ret;
	cmd.slave = smb_wbtemp1;
	cmd.cmd = 0x0;
	cmd.data.word_ptr = &ret;
	if (ioctl(iosmb,SMB_READW,&cmd) == -1) {
/*		strcpy(buf, "ioctl(");
		strcat(buf, smb_devfile + 5);
		strcat(buf, ":temp1)");
		perror(buf);
		exit(-1); */
		ret = 0xFFFF;
	}
	return (int) (ret & 0xFFFF);
}

static int ReadTemp2()
{
	struct smbcmd cmd;
	short ret;
	cmd.slave = smb_wbtemp2;
	cmd.cmd = 0x0;
	cmd.data.word_ptr = &ret;
	if (ioctl(iosmb,SMB_READW,&cmd) == -1) {
/*		strcpy(buf, "ioctl(");
		strcat(buf, smb_devfile + 5);
		strcat(buf, ":temp2)");
		perror(buf);
		exit(-1); */
		ret = 0xFFFF;
	}
	return (int) (ret & 0xFFFF);
}

struct lm_methods method_smb = {
  OpenIO, CloseIO, ReadByte, WriteByte, ReadTemp1, ReadTemp2
};

#endif
