/************************************************
	Subroutines to get Mother Board Information

	Information related to Winbond W83781D Chip
		and National Semiconductor LM78/LM79 Chips
	by Alex van Kaam

	Information for VIA VT82C686A/B
	by ":p araffin.(Yoneya)", MANY THANKS!!

	Information for SMBus access
	by Linux lm_sensor homepage, MANY THANKS!!
		http://www.netroedge.com/~lm78/

 ************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include "pci_pm.h"
#include "smbuses.h"
#include "methods.h"
#include "sensors.h"

/* external (global) data */
extern int debug_flag, fahrn_flag;
extern int TyanTigerMP_flag;
extern int isa_port_base;
extern int viahwm_base, smb_base;
extern int pm_smb_detected;
extern char *probe_request, *HWM_name[];
extern SENSOR *HWM_module[];

/* Access method functions, global */
extern LM_METHODS method_isa, method_smb, method_via;


static LM_METHODS *this_method = NULL;
static SENSOR *this_sensor = NULL;

static int probe_flag = SEARCH;


/* function declarations */
extern void TyanTigerMPinit(void);
int Probe_method(char);
int probe_HWMChip(LM_METHODS *, int);
int via_set(void);
int smb_set(int, int);
int isa_set(int);
int InitMBInfo(char);
int getTemp(float *, float *, float *);
int getVolt(float *, float *, float *, float *, float *, float *, float *);
int getFanSp(int *, int *, int *);


/*----------------------
	Detecting HWM Chip
  ----------------------*/

int Probe_method(char method)
{
	int n;

	if (method != 'I') {
		pm_smb_detected = set_smbus_io(&viahwm_base, &smb_base);
	}
	if (method == 'V') {
  		if ((n = pm_smb_detected) == VIA686HWM) {
		/* VIA VT82C686 HWM is available */
			return via_set();
		} else if (n != -1) {
			fprintf(stderr, "No VIA686 HWM available!!\n");
			return 1;
		}
	} else if (method == 'S') {
  		if ((n = pm_smb_detected) > 0) {
		/* SMBus PowerManagement, hardware monitor exist ? */
			return smb_set(probe_flag, n);
		} else if (n != -1) {
			fprintf(stderr, "No SMBus HWM available!!\n");
			return 1;
		}
	} else if (method == 'I') {
		/* Just try ISA-IO method */
		if ((n = isa_set(probe_flag)) == 0) {
			return 0;
		} else if (n != -1) {
			fprintf(stderr, "No ISA-IO HWM available!!\n");
			return 1;
		}
	} else {
	/* No input method option: Try probing each HWM type */
	  	if ((probe_flag == SEARCH || probe_flag == c_via686)
					&& (n = pm_smb_detected) == VIA686HWM) {
			if (via_set() == 0)
				return 0;
			else if (n > 0)
				goto smb_chk;
		} else if ((n = pm_smb_detected) > 0) {
smb_chk:	if (smb_set(probe_flag, n) == 0)
				return 0;
			goto isa_chk;
		} else {
isa_chk:	if ((n = isa_set(probe_flag)) == 0)
				return 0;
			else if (n != -1) {
				fprintf(stderr, "No Hardware Monitor found!!\n");
				return 1;
			}
		}
	}
	return -1;
}

int via_set(void)
{
	if (debug_flag > 1)
		fprintf(stderr, ">>> Testing Reg's at VIA686 HWM <<<\n");

	if (probe_HWMChip(&method_via, c_via686) > 0) {
		this_method = &method_via;
		if (debug_flag)
			fprintf(stderr, "Using VIA686 HWM directly!!\n");
		return 0;
	} else {
		fprintf(stderr, "Something Wrong in detected VIA686 HWM!!\n");
		return 1;
	}
}

int smb_set(int probe, int chip)
{
	if (debug_flag > 1) {
#if !defined(LINUX) && defined(HAVE_SMBUS) && defined(SMBUS_IOCTL)
		fprintf(stderr, ">>> Testing Reg's at SMBus <<<\n");
#else		/* SMBus direct access routines */
		fprintf(stderr, ">>> Testing Reg's at SMBus <<<\n"\
			"[%s, IO-Base:0x%0X]\n", chk_smb_chip(chip), smb_base);
#endif
	}
	if (find_smb_dev() <= 0)
		goto ret1;

	if (probe_HWMChip(&method_smb, probe) > 0) {
		this_method = &method_smb;
		if (debug_flag) {
#if !defined(LINUX) && defined(HAVE_SMBUS) && defined(SMBUS_IOCTL)
			fprintf(stderr, "Using SMBus-ioctl access method[%s]!!\n",
				chk_smb_chip(chip));
#else		/* SMBus direct access routines */
			fprintf(stderr, "Using SMBus access method[%s]!!\n",
				chk_smb_chip(chip));
#endif
		}
		return 0;
	} else {
ret1:	if (debug_flag) {
			fprintf(stderr, "SMBus[%s] found, but No HWM available on it!!\n",
				chk_smb_chip(chip));
		}
		return 1;
	}
}

int isa_set(int probe)
{
	int n;

	if (debug_flag > 1)
		fprintf(stderr, ">>> Testing Reg's at ISA-IO <<<\n"\
			"[ISA Port IO-Base:0x%0X]\n", isa_port_base);

	if ((n = probe_HWMChip(&method_isa, probe)) > 0) {
		this_method = &method_isa;
		if (debug_flag)
			fprintf(stderr, "Using ISA-IO access method!!\n");
		return 0;
	} else if (n == 0)
		n = 1;

	return n;
}

int probe_HWMChip(LM_METHODS *method, int probe)
{
	int n, ret = 0;

	if (method->Open() != 0)
		return -1;

	/* module order is important !! */
	for (n = 0; HWM_module[n] != NULL; n++) {
		if ((probe == SEARCH || probe == n)
				&& HWM_module[n]->Probe(method) != 0) {
			this_sensor = HWM_module[n];
			ret = 1;
			break;
		}
	}

	method->Close();
	return ret;
}

int InitMBInfo(char method)
{
	int n;

 	/* this is TyanTigerMP specific treatment */
	if (TyanTigerMP_flag)
	 	TyanTigerMPinit();

	if (debug_flag > 1)
		fprintf(stderr, "Probe Request: %s\n", probe_request);

	for (n = 0; HWM_name[n] != NULL; n++) {
		if (strcmp(probe_request, HWM_name[n]) == 0) {
			probe_flag = n;
			break;
		}
	}

	if ((n = Probe_method(method)) != 0) {
		return n;
	}

	if (this_method && this_sensor) {
		if (debug_flag)
			printf("* %s found.\n", this_sensor->Name);
		return 0;
	} else {
		return -1;
	}
}


/*-------------------------
	Getting Temperatures
  -------------------------*/

#define traFahrn(x)	((x) * 1.8 + 32.0)

int getTemp(float *t1, float *t2, float *t3)
{
	float f;

	if (this_method->Open() != 0)
		return -1;

	*t1	= 0;
	*t2	= 0;
	*t3	= 0;

	if (this_sensor) {
		if ((f = this_sensor->Temp(this_method, 0)) != 0xFFFF)
			*t1	= f;
		if ((f = this_sensor->Temp(this_method, 1)) != 0xFFFF)
			*t2	= f;
		if(( f = this_sensor->Temp(this_method, 2)) != 0xFFFF)
			*t3	= f;
  	}
	if (this_sensor == &it87 || this_sensor == &via686) {
		f = *t1;
		*t1 = *t2;
		*t2 = *t3;
		*t3 = f;
	}
	if (fahrn_flag) {
		if (*t1 != 0.0)
			*t1 = traFahrn(*t1);
		if (*t2 != 0.0)
			*t2 = traFahrn(*t2);
		if (*t3 != 0.0)
			*t3 = traFahrn(*t3);
	}

	this_method->Close();
	return 0;
}

/*--------------------
	Getting Voltages
  --------------------*/

int getVolt(float *vc0, float *vc1,\
			float *v33, float *v50p, float *v50n,\
			float *v12p, float *v12n)
{
	float f;

	if (this_method->Open() != 0)
		return -1;

	*vc0	= 0;
	*vc1	= 0;
	*v33	= 0;
	*v50p	= 0;
	*v12p	= 0;
	*v12n	= 0;
	*v50n	= 0;

	if (this_sensor) {
		if ((f = this_sensor->Volt(this_method, 0)) != 0xFFFF)
			*vc0	= f;
		if ((f = this_sensor->Volt(this_method, 1)) != 0xFFFF)
			*vc1	= f;
		if ((f = this_sensor->Volt(this_method, 2)) != 0xFFFF)
			*v33	= f;
		if ((f = this_sensor->Volt(this_method, 3)) != 0xFFFF)
			*v50p	= f;
		if ((f = this_sensor->Volt(this_method, 4)) != 0xFFFF)
			*v12p	= f;
		if ((f = this_sensor->Volt(this_method, 5)) != 0xFFFF)
			*v12n	= f;
		if ((f = this_sensor->Volt(this_method, 6)) != 0xFFFF)
			*v50n	= f;
	}

	this_method->Close();
	return 0;
}

/*----------------------
	Getting Fan Speed
  ----------------------*/

int getFanSp(int *r1, int *r2, int *r3)
{

	if (this_method->Open() != 0)
		return -1;

	if (this_sensor) {
		int n;

		*r1	= 0;
		*r2	= 0;
		*r3	= 0;

		if ((n = this_sensor->FanRPM(this_method, 0)) != 0xFFFF)
			*r1	= n;
		if ((n = this_sensor->FanRPM(this_method, 1)) != 0xFFFF)
			*r2	= n;
		if ((n = this_sensor->FanRPM(this_method, 2)) != 0xFFFF)
			*r3	= n;

	}

	this_method->Close();
	return 0;
}
