
/* Looking for Power Management ChipSet by checking
	the PCI Configuration Register, by YRS 2001.08.

	Information on how to access SMBus is provided
	by ":p araffin.(Yoneya)", MANY THANKS!!

	Information for SMBus access and PCI chipset comes from
	the Linux lm_sensor codes: http://www.lm-sensors.nu
 */

#include <unistd.h>

#include "pci_pm.h"
#include "io_static.c"

#ifdef LINUX	/* LINUX */
/* counter for not calling iopl() multiply */
int iopl_counter = 0;
#else			/* FreeBSD */
/* file descripter for /dev/io */
int iofl;
#endif

int canSMBUS_base[] = {
	getSMBBA0,
	getSMBBA1,
	getSMBBA2,
	getSMBBA3,
	getSMBBA4,
	getSMBBA5,
	0 };


char *chk_smb_chip(int chip)
{
	char *comt;

	switch (chip) {
		case VIA686HWM:
		case VIA686SMB:
			comt = "VIA82C686(KT133/A)";
			break;
		case VIA596SMB:
			comt = "VIA596";
			break;
		case PIIX4SMB:
			comt = "IntelPIIX4(440BX/MX)";
			break;
		case SRVWSSSMB:
			comt = "ServerWorks(ServerSet Chipset)";
			break;
		case VIA8233SMB:
			comt = "VT8233/A(KT266/KT333)";
			break;
		case VIA8235SMB:
			comt = "VIA8231/VT8235";
			break;
		case ICH801SMB:
			comt = "Intel801/810/815(ICH/ICH2)";
			break;
		case AMD756SMB:
			comt = "AMD756/766/768/NVidia nForce";
			break;
		case ALI7101SMB:
			comt = "ALi M1535D+";
			break;
		default:
			comt = NULL;
	}
	return comt;
}

int chk_port_byte(int addr)
{
	int ret;
	if(OpenIO() == -1) return -1;
	ret = inb((u_short) addr);
	CloseIO();
	return ret;
}

int pci_pm_smb_prob(int *hwm_base, int *smb_base)
{
	u_int dat;
	u_char dev, fun;
	int ret = 0, inq_smbba = 0;

	if(OpenIO() == -1) return -1;
	*hwm_base = *smb_base = 0;
	for (dev = 0; dev < PCI_DEVM; ++dev) {
	  for (fun = 0; fun < PCI_FUNM; ++fun) {
		switch (pci_conf_read(PCI_BUSN, dev, fun, 0x00)) {
		case ID_VIA686:
			dat = pci_conf_read(PCI_BUSN, dev, fun, VIA686HWM_prob);
			if (dat && 0x00000001) {
				*hwm_base = 0xFFFE & \
					pci_conf_read(PCI_BUSN, dev, fun, VIA686HWM_base);
				ret = VIA686HWM;
			} else {
				ret = VIA686SMB;
			}
			inq_smbba = getSMBBA0;
			break;
		case ID_VIA596:
		case ID_VIA596B:
			ret = VIA596SMB;
			inq_smbba = getSMBBA0;
			break;
		case ID_VIA8235:
			ret = VIA8235SMB;
			inq_smbba = getSMBBA0;
			break;
		case ID_PIIX4:
		case ID_PII440MX:
		case ID_EFVIC66:
			ret = PIIX4SMB;
			inq_smbba = getSMBBA0;
			break;
		case ID_SRVWSB4:
		case ID_SRVWSB5:
			ret = SRVWSSSMB;
			inq_smbba = getSMBBA0;
			break;
		case ID_VIA8233:
		case ID_VIA8233A:
		case ID_VIA8233a:
		case ID_VIA8233C:
			ret = VIA8233SMB;
			inq_smbba = getSMBBA1;
			break;
		case ID_I801AA:
		case ID_I801AB:
		case ID_I801BA:
		case ID_I801CA:
		case ID_I801DB:
			ret = ICH801SMB;
			inq_smbba = getSMBBA2;
			break;
		case ID_AMD756:
		case ID_AMD766:
		case ID_AMD768:
			ret = AMD756SMB;
			inq_smbba = getSMBBA3;
			break;
		case ID_NFORCE:
			ret = AMD756SMB;
			inq_smbba = getSMBBA4;
			break;
		case ID_ALI7101:
			ret = ALI7101SMB;
			inq_smbba = getSMBBA5;
			break;
		default:
		}
		if (ret) {
			*smb_base = pci_conf_readw(PCI_BUSN, dev, fun, (u_char) inq_smbba);
			if (ret == AMD756SMB) {
				*smb_base &= 0xFF00;
				if (inq_smbba == getSMBBA3)
					*smb_base += AMD_SMBOFF;
			} else if (ret == ALI7101SMB) {
				*smb_base &= 0xFFE0;
			} else {
				*smb_base &= 0xFFF0;
			}
			goto ending;
		}
	  }
	}
ending:
	if (*smb_base < 0x100) {	/* not so confident but ... */
		ret = 0;
	}
	CloseIO();
	return ret;
}

int pci_smb_prob(int smb_base[], u_int chip_id[])
{
	u_int chip, dat, addr;
	u_char dev, fun;
	int i, n = 0;

	if(OpenIO() == -1) return -1;
	for (dev = 0; dev < PCI_DEVM; ++dev) {
	  for (fun = 0; fun < PCI_FUNM; ++fun) {
		chip = pci_conf_read(PCI_BUSN, dev, fun, 0x00);
		if (chip != 0xFFFFFFFF) {
		  for (i = 0; (addr = canSMBUS_base[i]) != 0; i++) {
			dat = pci_conf_readw(PCI_BUSN, dev, fun, (u_char) addr);
#ifdef PCI_LIST
	if (i == 0) {
		printf("bus=0x00:dev=0x%02X:fun=0x%02X ---> ", dev, fun);
		printf("chip=0x%08X [0x%02X] SMBase=0x%08X\n", chip, addr, dat);
	} else {
		printf("                                                ");
		printf("[0x%02X] SMBase=0x%08X\n", addr, dat);
	}
#endif
			if (dat != 0x0 && dat != 0xFFFFFFFF \
				&& (dat & 0x0F) == 1 && (dat & 0xFFF00000) == 0) {
				chip_id[n] = chip;
				if (chip == ID_AMD756 || chip == ID_AMD766 )
					smb_base[n] = (dat & 0xFF00) + AMD_SMBOFF;
				else
					smb_base[n] = (dat & 0xFFF0);
				if (inb(smb_base[n]) != 0xFF)
					++n;
			}
		  }
		}
	  }
	}
	CloseIO();
	return n;
}

/* Byte/Word read-out from PCI Configuration Registor.
	Here: Boundary considered.
 */

u_char pci_conf_readb(u_char bus, u_char dev, u_char fun, u_char reg)
{
	u_int dat, n;

	n = reg % 4;
	dat = pci_conf_read(bus, dev, fun, (reg & 0xFC));
	if (n == 0) {
		return (dat & 0xFF);
	} else {
		return ((dat >> (8*n)) & 0xFF);
	}
}

u_short pci_conf_readw(u_char bus, u_char dev, u_char fun, u_char reg)
{
	u_int dat, n;

	n = reg % 4;
	dat = pci_conf_read(bus, dev, fun, (reg & 0xFC));
	if (n == 0) {
		return (dat & 0xFFFF);
	} else if (n != 3) {
		return ((dat >> (8*n)) & 0xFFFF);
	} else {
		n = pci_conf_read(bus, dev, fun, (reg & 0xFC) + 4);
		return ((((dat >> (8*3)) & 0xFF) & (n << (8*1))) & 0xFFFF);
	}
}

/* PCI Configuration Registor

 31              23              15              7             0
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|1|0 0 0 0 0 0 0|  Bus Num      |Dev Num  |Func |Reg Num    |0 0|
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

 */

u_int pci_conf_read(u_char bus, u_char dev, u_char fun, u_char reg)
{
/* NOTE: do not check length of "bus, dev, fun, reg"!! */
/*		reg = (Reg Num) << 2  */
/*		reg & 0xFC  */
/*		fun & 0x07  */
/*		dev & 0x1F  */
/*		bus & 0xFF  */

	u_int pix = 0, dat;
	pix |= reg;
	pix |= (fun << 8);
	pix |= (dev << 11);
	pix |= (bus << 16);
	pix |= 0x80000000; 
	OUTl((unsigned int) PCI_CFIO, pix); WAIT;
	dat = inl((unsigned int) PCI_CREAD); WAIT;
	pix &= 0x7FFFFFFF; 
	OUTl((unsigned int) PCI_CFIO, pix); WAIT;
	return dat;
}
